"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.minimalTheme = exports.defaultTheme = exports.buildTheme = void 0;
const assert = require("assert");
const fs_1 = require("fs");
const path_1 = require("path");
const preact_1 = require("preact");
const preact_render_to_string_1 = require("preact-render-to-string");
const default_templates_1 = require("./default-templates");
const router_1 = require("./router");
const comment_1 = require("./comment");
const highlight_1 = require("./highlight");
const fs_2 = require("../utils/fs");
const STATIC_DIR = path_1.join(__dirname, "../../../static");
// Keep this small, it will be inlined on EVERY page.
const THEME_JS = fs_1.readFileSync(path_1.join(STATIC_DIR, "theme.js"), "utf-8").trim();
const writeFile = async (path, content) => {
    await fs_1.promises.mkdir(path_1.dirname(path), { recursive: true });
    await fs_1.promises.writeFile(path, content);
};
const copyFile = async (src, dest) => {
    await fs_1.promises.mkdir(path_1.dirname(dest), { recursive: true });
    await fs_1.promises.copyFile(src, dest);
};
function buildTheme(routerCtor, templates) {
    return async (app, project, outDir) => {
        const start = Date.now();
        const router = new routerCtor(project);
        const themeTemplates = { ...default_templates_1.DefaultTemplates, ...templates };
        // TODO: These ought to be configurable.
        const highlighter = await highlight_1.DoubleHighlighter.create("light-plus", "dark-plus");
        app.renderer.hooks.on("body.begin", () => (preact_1.createElement("script", { dangerouslySetInnerHTML: { __html: THEME_JS } })));
        const pages = [project];
        const tasks = [];
        function boundParseMarkdown(markdown, reflection) {
            let result = comment_1.replaceIncludes(app.options.getValue("includes"), markdown, app.logger);
            result = comment_1.replaceMedia(app.options.getValue("media"), result, app.logger, reflection, router);
            result = comment_1.parseMarkdown(markdown, reflection, router, highlighter);
            return result;
        }
        let page = pages.shift();
        while (page) {
            assert(router.hasOwnDocument(page));
            const path = router.getDocumentName(page);
            app.logger.verbose(`${page.getFullName()} ===> ${path}`);
            const content = "<!DOCTYPE html>\n" +
                preact_render_to_string_1.render(preact_1.createElement(themeTemplates.Page, { hooks: app.renderer.hooks, reflection: page, router: router, parseMarkdown: boundParseMarkdown, templates: themeTemplates, highlighter: highlighter }), null, { pretty: false });
            tasks.push(writeFile(path_1.join(outDir, path), content));
            if (page.isContainer()) {
                pages.push(...page.children.filter((child) => router.hasOwnDocument(child)));
            }
            page = pages.shift();
        }
        // Copy static files
        tasks.push(copyFile(path_1.join(STATIC_DIR, "style.css"), path_1.join(outDir, router.getAssetDirectory(), "style.css")));
        // Write theme css
        tasks.push(writeFile(path_1.join(outDir, router.getAssetDirectory(), "theme.css"), highlighter.getStyles()));
        if (app.options.getValue("media")) {
            tasks.push(fs_2.copy(app.options.getValue("media"), path_1.join(outDir, router.getMediaDirectory())));
        }
        await Promise.all(tasks);
        app.logger.verbose(`[Perf] Theme output took ${Date.now() - start}ms`);
    };
}
exports.buildTheme = buildTheme;
exports.defaultTheme = buildTheme(router_1.ThemeRouter);
exports.minimalTheme = buildTheme(router_1.MinimalThemeRouter);
//# sourceMappingURL=theme.js.map